@@ -1,6 +1,6 @@
 // This file is part of BOINC.
 // http://boinc.berkeley.edu
-// Copyright (C) 2008 University of California
+// Copyright (C) 2020 University of California
 //
 // BOINC is free software; you can redistribute it and/or modify it
 // under the terms of the GNU Lesser General Public License
@@ -22,12 +22,6 @@
 #ifdef _WIN32
 #include "boinc_win.h"
 #include "win_util.h"
-#define unlink   _unlink
-#ifdef _MSC_VER
-#define snprintf _snprintf
-#define strdup   _strdup
-#define getcwd   _getcwd
-#endif
 #else
 #include "config.h"
 #if HAVE_SCHED_SETSCHEDULER && defined (__linux__)
@@ -55,13 +49,6 @@
 #include <process.h>
 #endif
 
-#if (defined (__APPLE__) && (defined(__i386__) || defined(__x86_64__)))
-#include <mach-o/loader.h>
-#include <mach-o/fat.h>
-#include <mach/machine.h>
-#include <libkern/OSByteOrder.h>
-#endif
-
 #if(!defined (_WIN32) && !defined (__EMX__))
 #include <fcntl.h>
 #endif
@@ -287,6 +274,9 @@ void ACTIVE_TASK::init_app_init_data(APP_INIT_DATA& aid) {
         FILE_REF& fref = avp->app_files[i];
         aid.app_files.push_back(string(fref.file_name));
     }
+    aid.no_priority_change = cc_config.no_priority_change;
+    aid.process_priority = cc_config.process_priority;
+    aid.process_priority_special = cc_config.process_priority_special;
 }
 
 // write the app init file.
@@ -337,7 +327,7 @@ static int create_dirs_for_logical_name(
     safe_strcpy(dir_path, slot_dir);
     char* p = buf;
     while (1) {
-        char* q = strstr(p, "/");
+        char* q = strchr(p, '/');
         if (!q) break;
         *q = 0;
         safe_strcat(dir_path, "/");
@@ -351,7 +341,7 @@ static int create_dirs_for_logical_name(
 
 static void prepend_prefix(APP_VERSION* avp, char* in, char* out, int len) {
     if (strlen(avp->file_prefix)) {
-        snprintf(out, len, "%s/%s", avp->file_prefix, in);
+        snprintf(out, len, "%.16s/%.200s", avp->file_prefix, in);
     } else {
         strlcpy(out, in, len);
     }
@@ -467,7 +457,7 @@ int ACTIVE_TASK::link_user_files() {
 }
 
 int ACTIVE_TASK::copy_output_files() {
-    char slotfile[256], projfile[256], open_name[256];
+    char slotfile[MAXPATHLEN], projfile[256], open_name[256];
     unsigned int i;
     for (i=0; i<result->output_files.size(); i++) {
         FILE_REF& fref = result->output_files[i];
@@ -476,7 +466,7 @@ int ACTIVE_TASK::copy_output_files() {
         prepend_prefix(
             app_version, fref.open_name, open_name, sizeof(open_name)
         );
-        snprintf(slotfile, sizeof(slotfile), "%s/%s", slot_dir, open_name);
+        snprintf(slotfile, sizeof(slotfile), "%.*s/%.*s", DIR_LEN, slot_dir, FILE_LEN, open_name);
         get_pathname(fip, projfile, sizeof(projfile));
         int retval = boinc_rename(slotfile, projfile);
         // the rename fails if the output file isn't there.
@@ -509,22 +499,22 @@ static int get_priority(bool is_high_priority) {
     int p = is_high_priority?cc_config.process_priority_special:cc_config.process_priority;
 #ifdef _WIN32
     switch (p) {
-    case 0: return IDLE_PRIORITY_CLASS;
-    case 1: return BELOW_NORMAL_PRIORITY_CLASS;
-    case 2: return NORMAL_PRIORITY_CLASS;
-    case 3: return ABOVE_NORMAL_PRIORITY_CLASS;
-    case 4: return HIGH_PRIORITY_CLASS;
-    case 5: return REALTIME_PRIORITY_CLASS;
+    case CONFIG_PRIORITY_LOWEST: return IDLE_PRIORITY_CLASS;
+    case CONFIG_PRIORITY_LOW: return BELOW_NORMAL_PRIORITY_CLASS;
+    case CONFIG_PRIORITY_NORMAL: return NORMAL_PRIORITY_CLASS;
+    case CONFIG_PRIORITY_HIGH: return ABOVE_NORMAL_PRIORITY_CLASS;
+    case CONFIG_PRIORITY_HIGHEST: return HIGH_PRIORITY_CLASS;
+    case CONFIG_PRIORITY_REALTIME: return REALTIME_PRIORITY_CLASS;
     }
     return is_high_priority ? BELOW_NORMAL_PRIORITY_CLASS : IDLE_PRIORITY_CLASS;
 #else
     switch (p) {
-    case 0: return PROCESS_IDLE_PRIORITY;
-    case 1: return PROCESS_MEDIUM_PRIORITY;
-    case 2: return PROCESS_NORMAL_PRIORITY;
-    case 3: return PROCESS_ABOVE_NORMAL_PRIORITY;
-    case 4: return PROCESS_HIGH_PRIORITY;
-    case 5: return PROCESS_REALTIME_PRIORITY;
+    case CONFIG_PRIORITY_LOWEST: return PROCESS_IDLE_PRIORITY;
+    case CONFIG_PRIORITY_LOW: return PROCESS_MEDIUM_PRIORITY;
+    case CONFIG_PRIORITY_NORMAL: return PROCESS_NORMAL_PRIORITY;
+    case CONFIG_PRIORITY_HIGH: return PROCESS_ABOVE_NORMAL_PRIORITY;
+    case CONFIG_PRIORITY_HIGHEST: return PROCESS_HIGH_PRIORITY;
+    case CONFIG_PRIORITY_REALTIME: return PROCESS_REALTIME_PRIORITY;
     }
     return is_high_priority ? PROCESS_MEDIUM_PRIORITY : PROCESS_IDLE_PRIORITY;
 #endif
@@ -843,6 +833,18 @@ int ACTIVE_TASK::start(bool test) {
     pid = process_info.dwProcessId;
     process_handle = process_info.hProcess;
     CloseHandle(process_info.hThread);  // thread handle is not used
+
+#ifdef _WIN64
+    // if host has multiple processor groups (i.e. > 64 processors)
+    // see which one was used for this job, and show it
+    //
+    if (log_flags.task_debug && gstate.host_info.n_processor_groups > 0) {
+        msg_printf(wup->project, MSG_INFO, 
+            "[task_debug] task is running in processor group %d",
+            get_processor_group(process_handle)
+        );
+    }
+#endif
 #elif defined(__EMX__)
 
     char* argv[100];
@@ -979,10 +981,6 @@ int ACTIVE_TASK::start(bool test) {
     }
     app_client_shm.reset_msgs();
 
-#if (defined (__APPLE__) && (defined(__i386__) || defined(__x86_64__)))
-    // PowerPC apps emulated on i386 Macs crash if running graphics
-    powerpc_emulated_on_i386 = ! is_native_i386_app(exec_path);
-#endif
     if (cc_config.run_apps_manually) {
         pid = getpid();     // use the client's PID
         set_task_state(PROCESS_EXECUTING, "start");
@@ -1095,7 +1093,7 @@ int ACTIVE_TASK::start(bool test) {
                 struct sched_param sp;
                 sp.sched_priority = 0;
                 if (sched_setscheduler(0, SCHED_IDLE, &sp)) {
-                    perror("sched_setscheduler");
+                    perror("app_start sched_setscheduler(SCHED_IDLE)");
                 }
             }
 #endif
@@ -1108,7 +1106,7 @@ int ACTIVE_TASK::start(bool test) {
         if (test) {
             strcpy(buf, exec_path);
         } else {
-            snprintf(buf, sizeof(buf), "../../%s", exec_path);
+            snprintf(buf, sizeof(buf), "../../%.1024s", exec_path);
         }
         if (g_use_sandbox) {
             char switcher_path[MAXPATHLEN];
@@ -1167,7 +1165,7 @@ error:
     //
     gstate.input_files_available(result, true);
     char err_msg[4096];
-    snprintf(err_msg, sizeof(err_msg), "couldn't start app: %s", buf);
+    snprintf(err_msg, sizeof(err_msg), "couldn't start app: %.256s", buf);
     gstate.report_result_error(*result, err_msg);
     if (log_flags.task_debug) {
         msg_printf(wup->project, MSG_INFO,
@@ -1238,83 +1236,6 @@ int ACTIVE_TASK::resume_or_start(bool first_time) {
     return 0;
 }
 
-#if (defined (__APPLE__) && (defined(__i386__) || defined(__x86_64__)))
-
-union headeru {
-    fat_header fat;
-    mach_header mach;
-};
-
-// Read the mach-o headers to determine the architectures
-// supported by executable file.
-// Returns 1 if application can run natively on i386 / x86_64 Macs,
-// else returns 0.
-//
-int ACTIVE_TASK::is_native_i386_app(char* exec_path) {
-    FILE *f;
-    int retval = 0;
-    
-    headeru myHeader;
-    fat_arch fatHeader;
-    
-    uint32_t n, i, len;
-    uint32_t theMagic;
-    integer_t theType;
-    
-    f = boinc_fopen(exec_path, "rb");
-    if (!f) {
-        return retval;          // Should never happen
-    }
-    
-    myHeader.fat.magic = 0;
-    myHeader.fat.nfat_arch = 0;
-    
-    fread(&myHeader, 1, sizeof(fat_header), f);
-    theMagic = myHeader.mach.magic;
-    switch (theMagic) {
-    case MH_CIGAM:
-    case MH_MAGIC:
-    case MH_MAGIC_64:
-    case MH_CIGAM_64:
-       theType = myHeader.mach.cputype;
-        if ((theMagic == MH_CIGAM) || (theMagic == MH_CIGAM_64)) {
-            theType = OSSwapInt32(theType);
-        }
-        if ((theType == CPU_TYPE_I386) || (theType == CPU_TYPE_X86_64)) {
-            retval = 1;        // Single-architecture i386or x86_64 file
-        }
-        break;
-    case FAT_MAGIC:
-    case FAT_CIGAM:
-        n = myHeader.fat.nfat_arch;
-        if (theMagic == FAT_CIGAM) {
-            n = OSSwapInt32(myHeader.fat.nfat_arch);
-        }
-           // Multiple architecture (fat) file
-        for (i=0; i<n; i++) {
-            len = fread(&fatHeader, 1, sizeof(fat_arch), f);
-            if (len < sizeof(fat_arch)) {
-                break;          // Should never happen
-            }
-            theType = fatHeader.cputype;
-            if (theMagic == FAT_CIGAM) {
-                theType = OSSwapInt32(theType);
-            }
-            if ((theType == CPU_TYPE_I386) || (theType == CPU_TYPE_X86_64)) {
-                retval = 1;
-                break;
-            }
-        }
-        break;
-    default:
-        break;
-    }
-
-    fclose (f);
-    return retval;
-}
-#endif
-
 // The following runs "test_app" and sends it various messages.
 // Used for testing the runtime system.
 //
@@ -1355,7 +1276,7 @@ void run_test_app() {
     at.max_mem_usage = 1e14;
     safe_strcpy(at.slot_dir, ".");
 
-#if 1
+#if 0
     // test file copy
     //
     ASYNC_COPY* ac = new ASYNC_COPY;
