@@ -30,10 +30,6 @@
 #include <sys/types.h>
 #endif
 
-#ifdef _MSC_VER
-#define snprintf _snprintf
-#endif
-
 #include "md5_file.h"
 #include "crypt.h"
 #include "str_replace.h"
@@ -192,7 +188,7 @@ int FILE_INFO::verify_file(
     // see if we need to unzip it
     //
     if (download_gzipped && !boinc_file_exists(pathname)) {
-        char gzpath[MAXPATHLEN];
+        char gzpath[MAXPATHLEN+16];
         snprintf(gzpath, sizeof(gzpath), "%s.gz", pathname);
         if (boinc_file_exists(gzpath) ) {
             if (allow_async && nbytes > ASYNC_FILE_THRESHOLD) {
@@ -417,7 +413,7 @@ bool CLIENT_STATE::create_and_delete_pers_file_xfers() {
                 // If this was a compressed download, rename .gzt to .gz
                 //
                 if (fip->download_gzipped) {
-                    char path[MAXPATHLEN], from_path[MAXPATHLEN], to_path[MAXPATHLEN];
+                    char path[MAXPATHLEN], from_path[MAXPATHLEN+16], to_path[MAXPATHLEN+16];
                     get_pathname(fip, path, sizeof(path));
                     snprintf(from_path, sizeof(from_path), "%s.gzt", path);
                     snprintf(to_path, sizeof(to_path), "%s.gz", path);
