@@ -1,6 +1,6 @@
 // This file is part of BOINC.
 // http://boinc.berkeley.edu
-// Copyright (C) 2018 University of California
+// Copyright (C) 2019 University of California
 //
 // BOINC is free software; you can redistribute it and/or modify it
 // under the terms of the GNU Lesser General Public License
@@ -22,6 +22,8 @@
 #ifdef __APPLE__
 #include <Carbon/Carbon.h>
 #include <libproc.h>
+#include "sandbox.h"
+#include "mac_branding.h"
 #endif
 
 #ifdef _WIN32
@@ -51,10 +53,6 @@
 #endif
 #endif
 
-#ifdef _MSC_VER
-#define snprintf _snprintf
-#endif
-
 #include "error_numbers.h"
 #include "filesys.h"
 #include "network.h"
@@ -166,8 +164,8 @@ static void handle_get_disk_usage(GUI_RPC_CONN& grc) {
         );
     }
 
-    dir_size(".", boinc_non_project, false);
-    dir_size("locale", size, false);
+    dir_size_alloc(".", boinc_non_project, false);
+    dir_size_alloc("locale", size, false);
     boinc_non_project += size;
 #ifdef __APPLE__
     if (gstate.launched_by_manager) {
@@ -177,9 +175,13 @@ static void handle_get_disk_usage(GUI_RPC_CONN& grc) {
         OSStatus err = noErr;
         
         retval = proc_pidpath(getppid(), path, sizeof(path));
-        if (retval <= 0) err = fnfErr;
-        if (! err) dir_size(path, manager_size, true);
-        if (! err) boinc_non_project += manager_size;
+        if (retval <= 0) {
+            err = fnfErr;
+        }
+        if (!err) {
+            dir_size_alloc(path, manager_size, true);
+            boinc_non_project += manager_size;
+        }
     }
 #endif
     boinc_total = boinc_non_project;
@@ -746,7 +748,7 @@ static void handle_get_project_init_status(GUI_RPC_CONN& grc) {
     //
     for (unsigned i=0; i<gstate.projects.size(); i++) { 
         PROJECT* p = gstate.projects[i]; 
-        if (!strcmp(p->master_url, gstate.project_init.url)) { 
+        if (urls_match(p->master_url, gstate.project_init.url)) { 
             gstate.project_init.remove(); 
             break; 
         } 
@@ -756,15 +758,11 @@ static void handle_get_project_init_status(GUI_RPC_CONN& grc) {
         "<get_project_init_status>\n"
         "    <url>%s</url>\n"
         "    <name>%s</name>\n"
-        "    <team_name>%s</team_name>\n"
-        "    <setup_cookie>%s</setup_cookie>\n"
         "    %s\n"
         "    %s\n"
         "</get_project_init_status>\n",
         gstate.project_init.url,
         gstate.project_init.name,
-        gstate.project_init.team_name,
-        gstate.project_init.setup_cookie,
         strlen(gstate.project_init.account_key)?"<has_account_key/>":"",
         gstate.project_init.embedded?"<embedded/>":""
     );
@@ -1332,6 +1330,258 @@ static void handle_get_daily_xfer_history(GUI_RPC_CONN& grc) {
     daily_xfer_history.write_xml(grc.mfout);
 }
 
+#ifdef __APPLE__
+static void stop_graphics_app(pid_t thePID, 
+                            long iBrandID, 
+                            char current_dir[], 
+                            char switcher_path[], 
+                            string theScreensaverLoginUser, 
+                            GUI_RPC_CONN& grc
+                            ) {
+    char* argv[16];
+    int argc;
+    char screensaverLoginUser[256];
+    int newPID = 0;
+    int retval;
+
+    if (g_use_sandbox) {
+        char pidString[10];
+        
+        snprintf(pidString, sizeof(pidString), "%d", thePID);
+    #if 1
+        argv[0] = const_cast<char*>(SWITCHER_FILE_NAME);
+        argv[1] = saverName[iBrandID];
+        argv[2] = "-kill_gfx";
+        argv[3] = pidString;
+        argc = 4;
+    #else 
+        argv[0] = const_cast<char*>(SWITCHER_FILE_NAME);
+        argv[1] = "/bin/kill";
+        argv[2] = "-kill";
+        argv[3] = (char *)pidString;
+        argc = 4;
+    #endif
+        if (!theScreensaverLoginUser.empty()) {
+            argv[argc++] = "--ScreensaverLoginUser";
+            safe_strcpy(screensaverLoginUser, theScreensaverLoginUser.c_str());
+            argv[argc++] = screensaverLoginUser;
+        }
+        argv[argc] = 0;
+
+        retval = run_program(
+            current_dir, switcher_path,
+            argc, argv, 0, newPID
+        );
+    } else {
+        retval = kill_program(thePID);
+    }
+    if (retval) {
+        grc.mfout.printf("<error>attempt to kill graphics app failed</error>\n");
+        return;
+    }
+    grc.mfout.printf("<success/>\n");
+    return;
+}
+#endif
+
+// start, stop or get status of a graphics app on behalf of the screensaver.
+// (needed for Mac OS X 10.15+; "stop & "test" are used for Mac OS X 10.13+)
+//
+// <slot>n</slot> { <run/> | <runfullscreen/> }
+// <graphics_pid>p</graphics_pid> { <stop/> | <test/> }
+//
+// n is the slot number:
+//   if slot = -1, start the default screensaver
+// p is the process id to stop
+//   test returns 0 for the pid if it has exited, else returns the child's pid
+//
+static void handle_run_graphics_app(GUI_RPC_CONN& grc) {
+#ifndef __APPLE__
+    grc.mfout.printf("<error>run_graphics_app RPC is currently available only on Mac OS</error>\n");
+#else
+    bool run = false;
+    bool runfullscreen = false;
+    bool stop = false;
+    bool test = false;
+    int slot = -2, retval;
+    pid_t p;
+    char* argv[16];
+    int argc;
+    int thePID = 0;
+    FILE *f;
+    long iBrandID;
+    string theScreensaverLoginUser;
+    char screensaverLoginUser[256];
+    char switcher_path[MAXPATHLEN];
+    char *execName, *execPath;
+    char current_dir[MAXPATHLEN];
+    char *execDir;
+    int newPID = 0;
+    ACTIVE_TASK* atp = NULL;
+    char cmd[256];
+    
+    while (!grc.xp.get_tag()) {
+        if (grc.xp.match_tag("/run_graphics_app")) break;
+        if (grc.xp.parse_int("slot", slot)) continue;
+        if (grc.xp.parse_bool("run", run)) continue;
+        if (grc.xp.parse_bool("runfullscreen", runfullscreen)) continue;
+        if (grc.xp.parse_bool("stop", stop)) continue;
+        if (grc.xp.parse_bool("test", test)) continue;
+        if (grc.xp.parse_int("graphics_pid", thePID)) continue;
+        if (grc.xp.parse_string("ScreensaverLoginUser", theScreensaverLoginUser)) continue;
+    }
+    
+    if (stop || test) {
+        if (thePID < 1) {
+            grc.mfout.printf("<error>missing or invalid process id</error>\n");
+            return;
+        }
+    } else if (run || runfullscreen) {
+        if (slot < -1) {
+            grc.mfout.printf("<error>missing or invalid slot</error>\n");
+            return;
+        }
+    } else {
+        grc.mfout.printf("<error>missing or invalid operation</error>\n");
+        return;
+    }
+
+    if (test) {
+        // returns 0 for the pid if it has exited, else returns the child's pid
+        p = 0;
+        snprintf(cmd, sizeof(cmd), "ps -p %d -o pid", thePID);
+        f = popen(cmd, "r");
+        if (f) {
+            fgets(cmd, sizeof(cmd), f); // Skip the header line
+            fscanf(f, "%d", &p);
+            pclose(f);
+            grc.mfout.printf(
+                "<graphics_pid>%d</graphics_pid>\n<success/>\n",
+                p
+            );
+        }
+        return;
+    }
+
+    // For branded installs, the Mac installer put a branding file in our data directory
+    iBrandID = 0;   // Default value
+    f = fopen("/Library/Application Support/BOINC Data/Branding", "r");
+    if (f) {
+        fscanf(f, "BrandId=%ld\n", &iBrandID);
+        fclose(f);
+    }
+    if ((iBrandID < 0) || (iBrandID > (NUMBRANDS-1))) {
+        iBrandID = 0;
+    }
+
+    getcwd(current_dir, sizeof(current_dir));
+
+    if (g_use_sandbox) {
+        snprintf(switcher_path, sizeof(switcher_path), 
+            "%s/%s/%s",
+            current_dir, SWITCHER_DIR, SWITCHER_FILE_NAME
+        );
+    }
+
+    if (stop) {
+        stop_graphics_app(thePID, iBrandID, current_dir, switcher_path, 
+                            theScreensaverLoginUser, grc);
+        grc.mfout.printf("<success/>\n");
+        return;
+    }
+
+    if (slot == -1) {
+        // start boincscr
+        //
+        execPath = (char*)"./boincscr";
+        execName = (char*)"boincscr";
+        execDir = current_dir;
+    } else {   // if (slot != -1)
+        // start a graphics app
+        //
+        atp = gstate.active_tasks.lookup_slot(slot);
+        if (!atp) {
+            grc.mfout.printf("<error>no job in slot</error>\n");
+            return;
+        }
+        if (atp->scheduler_state != CPU_SCHED_SCHEDULED) {
+            grc.mfout.printf("<error>job not running</error>\n");
+            return;
+        }
+        if (!strlen(atp->app_version->graphics_exec_path)) {
+            grc.mfout.printf("<error>job has no graphics app</error>\n");
+            return;
+        }
+        
+        execPath = atp->app_version->graphics_exec_path;
+        execName = atp->app_version->graphics_exec_file;
+        execDir = atp->slot_path;
+    }
+
+    if (g_use_sandbox) {
+        if (slot == -1) {
+            argv[0] = const_cast<char*>(SWITCHER_FILE_NAME);
+            argv[1] = execDir;
+            argv[2] = saverName[iBrandID];
+            argv[3] = "-default_gfx";
+            argv[4] = "boincscr";
+            argc = 5;
+        } else {
+            char theSlot[10];
+            sprintf(theSlot, "%d", slot);
+            argv[0] = const_cast<char*>(SWITCHER_FILE_NAME);
+            argv[1] = execDir;
+            argv[2] = saverName[iBrandID];
+            argv[3] = "-launch_gfx";
+            argv[4] = (char *)theSlot;
+            argc = 5;
+        }
+        
+        if (runfullscreen) {
+            argv[argc++] = "--fullscreen";
+        }
+        if (!theScreensaverLoginUser.empty()) {
+            argv[argc++] = "--ScreensaverLoginUser";
+            safe_strcpy(screensaverLoginUser, theScreensaverLoginUser.c_str());
+            argv[argc++] = screensaverLoginUser;
+        }
+        argv[argc] = 0;
+
+        retval = run_program(
+            execDir, switcher_path,
+            argc, argv, 0, newPID
+        );
+    } else {    // not g_use_sandbox
+        argv[0] = execName;
+        if (runfullscreen) {
+            argv[1] = (char*)"--fullscreen";
+            argc = 2;
+        } else {
+            argc = 1;
+        }
+        if (!theScreensaverLoginUser.empty()) {
+            argv[argc++] = "--ScreensaverLoginUser";
+            safe_strcpy(screensaverLoginUser, theScreensaverLoginUser.c_str());
+            argv[argc++] = screensaverLoginUser;
+        }
+        argv[argc] = 0;
+        retval = run_program(
+            execDir, execPath,
+            argc, argv, 0, newPID
+        );
+    }
+    
+    if (retval) {
+        grc.mfout.printf("<error>couldn't run graphics app</error>\n");
+        stop_graphics_app(thePID, iBrandID, current_dir, switcher_path, 
+                            theScreensaverLoginUser, grc);
+    } else {
+        grc.mfout.printf("<success/>\n");
+    }
+    return;
+#endif  // __APPLE__
+}
+
 // We use a different authentication scheme for HTTP because
 // each request has its own connection.
 // Send clients an "authentication ID".
@@ -1349,7 +1599,7 @@ vector<AUTH_INFO> auth_infos;
 // check HTTP authentication info
 //
 bool valid_auth(int id, long seqno, char* hash, char* request) {
-    char buf[256], my_hash[256];
+    char buf[1024], my_hash[256];
     //printf("valid_auth: id %d seqno %ld hash %s\n", id, seqno, hash);
     for (unsigned int i=0; i<auth_infos.size(); i++) {
         AUTH_INFO& ai = auth_infos[i];
@@ -1377,7 +1627,7 @@ void handle_get_auth_id(MIOFILE& fout) {
     AUTH_INFO ai;
     ai.id = id++;
     ai.seqno = 0;
-    make_random_string(ai.salt);
+    make_secure_random_string(ai.salt);
     auth_infos.push_back(ai);
     fout.printf("<auth_id>%d</auth_id>\n<auth_salt>%s</auth_salt>\n", ai.id, ai.salt);
 }
@@ -1388,15 +1638,15 @@ static bool authenticated_request(char* buf) {
     int auth_id;
     long auth_seqno;
     char auth_hash[256];
-    char* p = strstr(buf, "Auth-ID: ");
+    const char* p = strcasestr(buf, "Auth-ID: ");
     if (!p) return false;
     int n = sscanf(p+strlen("Auth-ID: "), "%d", &auth_id);
     if (n != 1) return false;
-    p = strstr(buf, "Auth-Seqno: ");
+    p = strcasestr(buf, "Auth-Seqno: ");
     if (!p) return false;
     n = sscanf(p+strlen("Auth-Seqno: "), "%ld", &auth_seqno);
     if (n != 1) return false;
-    p = strstr(buf, "Auth-Hash: ");
+    p = strcasestr(buf, "Auth-Hash: ");
     if (!p) return false;
     n = sscanf(p+strlen("Auth-Hash: "), "%64s", auth_hash);
     if (n != 1) return false;
@@ -1599,6 +1849,7 @@ GUI_RPC gui_rpcs[] = {
     GUI_RPC("project_reset", handle_project_reset,                  true,   true,   false),
     GUI_RPC("project_update", handle_project_update,                true,   true,   false),
     GUI_RPC("retry_file_transfer", handle_retry_file_transfer,      true,   true,   false),
+    GUI_RPC("run_graphics_app", handle_run_graphics_app,            true,   true,   false),
 };
 
 // return nonzero only if we need to close the connection
@@ -1606,8 +1857,14 @@ GUI_RPC gui_rpcs[] = {
 static int handle_rpc_aux(GUI_RPC_CONN& grc) {
     int retval = 0;
     grc.mfin.init_buf_read(grc.request_msg);
-    if (grc.xp.get_tag()) return ERR_XML_PARSE;   // parse <boinc_gui_rpc_request>
-    if (grc.xp.get_tag()) return ERR_XML_PARSE;   // parse the request tag
+    if (grc.xp.get_tag()) {    // parse <boinc_gui_rpc_request>
+        grc.mfout.printf("<error>missing boing_gui_rpc_request tag</error>\n");
+        return 0;
+    }
+    if (grc.xp.get_tag()) {    // parse the request tag
+        grc.mfout.printf("<error>missing request</error>\n");
+        return 0;
+    }
     for (unsigned int i=0; i<sizeof(gui_rpcs)/sizeof(GUI_RPC); i++) {
         GUI_RPC& gr = gui_rpcs[i];
         if (!grc.xp.match_tag(gr.req_tag) && !grc.xp.match_tag(gr.alt_req_tag)) {
@@ -1639,13 +1896,21 @@ static bool is_http_post_request(char* buf) {
     if (!p) return false;
     p += strlen("Content-Length: ");
     int n = atoi(p);
-    p = strstr(p, "\r\n\r\n");
+    p = strstr(p, HTTP_HEADER_DELIM);
     if (!p) return false;
     p += 4;
     if ((int)strlen(p) < n) return false;
     return true;
 }
 
+// remove HTTP header from request
+//
+static void strip_http_header(char* buf) {
+    char* p = strstr(buf, HTTP_HEADER_DELIM);
+    p += 4;
+    strcpy_overlap(buf, p);
+}
+
 static bool is_http_get_request(char* buf) {
     return (strstr(buf, "GET") == buf);
 }
@@ -1662,16 +1927,24 @@ void GUI_RPC_CONN::http_error(const char* msg) {
 // - no ..
 //
 void GUI_RPC_CONN::handle_get() {
+    // no one is using this feature and it's a potential security risk,
+    // so disable it for now.
+    //
+    return http_error("HTTP/1.0 403 Access denied\n\nAccess denied\n");
+#if 0
     if (!cc_config.allow_gui_rpc_get) {
         return http_error("HTTP/1.0 403 Access denied\n\nAccess denied\n");
     }
 
     // get filename from GET /foo.html HTTP/1.1
+    // and make sure it's relative
     //
     char *p, *q=0;
     p = strchr(request_msg, '/');
     if (p) {
-        p++;
+        while (*p=='/') {
+            p++;
+        }
         q = strchr(p, ' ');
     }
 
@@ -1680,7 +1953,7 @@ void GUI_RPC_CONN::handle_get() {
     }
 
     *q = 0;
-    if (strstr(p, "..")) {
+    if (strstr(p, "..") || strchr(p, ':')) {
         return http_error("HTTP/1.0 400 Bad request\n\nBad HTTP request\n");
     }
     if (!ends_with(p, ".html")
@@ -1709,6 +1982,7 @@ void GUI_RPC_CONN::handle_get() {
     );
     send(sock, buf, (int)strlen(buf), 0);
     send(sock, file.c_str(), n, 0);
+#endif
 }
 
 // return nonzero only if we need to close the connection
@@ -1749,6 +2023,7 @@ int GUI_RPC_CONN::handle_rpc() {
             "Server: BOINC client\n"
             "Access-Control-Allow-Origin: *\n"
             "Access-Control-Allow-Methods: POST, GET, OPTIONS\n"
+            "Access-Control-Allow-Headers: *\n"
             "Content-Length: 0\n"
             "Keep-Alive: timeout=2, max=100\n"
             "Connection: Keep-Alive\n"
@@ -1774,6 +2049,7 @@ int GUI_RPC_CONN::handle_rpc() {
             got_auth1 = got_auth2 = true;
             auth_needed = false;
         }
+        strip_http_header(request_msg);
     } else {
         p = strchr(request_msg, 3);
         if (p) {
@@ -1834,6 +2110,9 @@ int GUI_RPC_CONN::handle_rpc() {
             "HTTP/1.1 200 OK\n"
             "Date: Fri, 31 Dec 1999 23:59:59 GMT\n"
             "Server: BOINC client\n"
+            "Access-Control-Allow-Origin: *\n"
+            "Access-Control-Allow-Methods: POST, GET, OPTIONS\n"
+            "Access-Control-Allow-Headers: *\n"
             "Connection: close\n"
             "Content-Type: text/xml; charset=utf-8\n"
             "Content-Length: %d\n\n"
