@@ -1,6 +1,6 @@
 // This file is part of BOINC.
 // http://boinc.berkeley.edu
-// Copyright (C) 2018 University of California
+// Copyright (C) 2020 University of California
 //
 // BOINC is free software; you can redistribute it and/or modify it
 // under the terms of the GNU Lesser General Public License
@@ -26,11 +26,14 @@
 #endif
 
 #ifdef __APPLE__
+#define VERBOSE 0
+
 #include <Carbon/Carbon.h>
 #include <sys/wait.h>
 #include <app_ipc.h>
 #include <malloc/malloc.h>
 #include <pthread.h>
+#include <sys/stat.h>
 
 extern pthread_mutex_t saver_mutex;
 #endif
@@ -48,6 +51,17 @@ extern pthread_mutex_t saver_mutex;
 #include "str_replace.h"
 #include "screensaver.h"
 
+#ifdef __APPLE__
+#undef BOINCTRACE
+#if VERBOSE
+#define BOINCTRACE print_to_log_file
+#else
+#define BOINCTRACE(...)
+#endif
+
+#define _T
+#endif
+
 // Platform specific application includes
 //
 #if   defined(_WIN32)
@@ -56,6 +70,7 @@ typedef HANDLE GFXAPP_ID;
 #define DataMgmtProcType DWORD WINAPI
 #elif defined(__APPLE__)
 #include "Mac_Saver_Module.h"
+#include "shmem.h"
 typedef int GFXAPP_ID;
 #define DataMgmtProcType void*
 #endif
@@ -79,6 +94,11 @@ typedef int GFXAPP_ID;
 // Flags for testing & debugging
 #define SIMULATE_NO_GRAPHICS 0
 
+RESULT* graphics_app_result_ptr = NULL;
+
+#ifdef __APPLE__
+pid_t* pid_from_shmem = NULL;
+#endif
 
 bool CScreensaver::is_same_task(RESULT* taska, RESULT* taskb) {
     if ((taska == NULL) || (taskb == NULL)) return false;
@@ -214,30 +234,54 @@ int CScreensaver::launch_screensaver(RESULT* rp, GFXAPP_ID& graphics_application
     if (strlen(rp->graphics_exec_path)) {
         // V6 Graphics
 #ifdef __APPLE__
-        // For sandbox security, use gfx_switcher to launch gfx app 
-        // as user boinc_project and group boinc_project.
-        //
-        // For unknown reasons, the graphics application exits with 
-        // "RegisterProcess failed (error = -50)" unless we pass its 
-        // full path twice in the argument list to execv.
-        char* argv[5];
-        argv[0] = "gfx_Switcher";
-        argv[1] = "-launch_gfx";
-        argv[2] = strrchr(rp->slot_path, '/');
-        if (*argv[2]) argv[2]++;    // Point to the slot number in ascii
-        
-        argv[3] = "--fullscreen";
-        argv[4] = 0;
-
-       retval = run_program(
-            rp->slot_path,
-            m_gfx_Switcher_Path,
-            4,
-            argv,
-            0,
-            graphics_application
-        );
+        if (gIsCatalina) {
+            // As of OS 10.15 (Catalina) screensavers can no longer:
+            //  - launch apps that run setuid or setgid
+            //  - launch apps downloaded from the Internet which have not been 
+            //    specifically approved by the  user via Gatekeeper.
+            // So instead of launching graphics apps via gfx_switcher, we send an 
+            // RPC to the client asking the client to launch them via gfx_switcher.
+            // See comments in gfx_switcher.cpp for a more detailed explanation.
+            // We have tested this on OS 10.13 High Sierra and it works there, too
+            //
+            retval = rpc->run_graphics_app("runfullscreen", rp->slot, gUserName);
+            for (int i=0; i<800; i++) {
+                boinc_sleep(0.01);      // Wait 8 seconds max
+                if (*pid_from_shmem != 0) {
+                    graphics_application = *pid_from_shmem;
+                    break;
+                }
+            }
+            // fprintf(stderr, "launch_screensaver got pid %d\n", graphics_application);
+            // Inform our helper app what we launched 
+            fprintf(m_gfx_Cleanup_IPC, "%d\n", graphics_application);
+            fflush(m_gfx_Cleanup_IPC);
+        } else {
+            // For sandbox security, use gfx_switcher to launch gfx app 
+            // as user boinc_project and group boinc_project.
+            //
+            // For unknown reasons, the graphics application exits with 
+            // "RegisterProcess failed (error = -50)" unless we pass its 
+            // full path twice in the argument list to execv.
+            char* argv[5];
+            argv[0] = "gfx_Switcher";
+            argv[1] = "-launch_gfx";
+            argv[2] = strrchr(rp->slot_path, '/');
+            if (*argv[2]) argv[2]++;    // Point to the slot number in ascii
+            
+            argv[3] = "--fullscreen";
+            argv[4] = 0;
 
+           retval = run_program(
+                rp->slot_path,
+                m_gfx_Switcher_Path,
+                4,
+                argv,
+                0,
+                graphics_application
+            );
+    }
+    
     if (graphics_application) {
         launchedGfxApp(rp->graphics_exec_path, graphics_application, rp->slot);
     }
@@ -262,59 +306,98 @@ int CScreensaver::launch_screensaver(RESULT* rp, GFXAPP_ID& graphics_application
 
 // Terminate any screensaver graphics application
 //
-int CScreensaver::terminate_v6_screensaver(GFXAPP_ID& graphics_application) {
+int CScreensaver::terminate_v6_screensaver(GFXAPP_ID& graphics_application, RESULT* rp) {
     int retval = 0;
+    int i;
 
 #ifdef __APPLE__
-    // Under sandbox security, use gfx_switcher to kill default gfx app 
-    // as user boinc_master and group boinc_master.  The man page for 
-    // kill() says the user ID of the process sending the signal must 
-    // match that of the target process, though in practice that seems 
-    // not to be true on the Mac.
-    
-    char current_dir[PATH_MAX];
-    char gfx_pid[16];
     pid_t thePID;
-    int i;
     
-    if (graphics_application == 0) return 0;
-    
-    // MUTEX may help prevent crashes when terminating an older gfx app which
-    // we were displaying using CGWindowListCreateImage under OS X >= 10.13
-    // Also prevents reentry when called from our other thread
-    pthread_mutex_lock(&saver_mutex);
+    if (gIsCatalina) {
+        // As of OS 10.15 (Catalina) screensavers can no longer launch apps
+        // that run setuid or setgid. So instead of killing graphics apps 
+        // via gfx_switcher, we send an RPC to the client asking the client 
+        // to kill them via switcher.
+        // We have tested this on OS 10.13 High Sierra and it works there, too
+        //
+        int ignore;
 
-    sprintf(gfx_pid, "%d", graphics_application);
-    getcwd( current_dir, sizeof(current_dir));
+        if (graphics_application == 0) return 0;
 
-    char* argv[4];
-    argv[0] = "gfx_switcher";
-    argv[1] = "-kill_gfx";
-    argv[2] = gfx_pid;
-    argv[3] = 0;
+        // MUTEX may help prevent crashes when terminating an older gfx app which
+        // we were displaying using CGWindowListCreateImage under OS X >= 10.13
+        // Also prevents reentry when called from our other thread
+        pthread_mutex_lock(&saver_mutex);
+
+        thePID = graphics_application;
+        // fprintf(stderr, "stopping pid %d\n", thePID);
+        retval = rpc->run_graphics_app("stop", thePID, gUserName);
+        //kill_program(graphics_application);
+
+        // Inform our helper app that we have stopped current graphics app 
+        fprintf(m_gfx_Cleanup_IPC, "0\n");
+        fflush(m_gfx_Cleanup_IPC);
 
-    retval = run_program(
-        current_dir,
-        m_gfx_Switcher_Path,
-        3,
-        argv,
-        0,
-        thePID
-    );
-    
-    if (graphics_application) {
         launchedGfxApp("", 0, -1);
+
+        for (i=0; i<200; i++) {
+            boinc_sleep(0.01);      // Wait 2 seconds max
+            if (HasProcessExited(graphics_application, ignore)) {
+                break;
+            }
+        }
+        pthread_mutex_unlock(&saver_mutex);
+    
+    } else {
+        // Under sandbox security, use gfx_switcher to kill default gfx app 
+        // as user boinc_master and group boinc_master (for default gfx app)
+        // or user boinc_project and group boinc_project (for project gfx 
+        // apps.) The man page for kill() says the user ID of the process 
+        // sending the signal must match that of the target process, though 
+        // in practice that seems not to be true on the Mac.
+        
+        char current_dir[PATH_MAX];
+        char gfx_pid[16];
+        
+        if (graphics_application == 0) return 0;
+        
+        // MUTEX may help prevent crashes when terminating an older gfx app which
+        // we were displaying using CGWindowListCreateImage under OS X >= 10.13
+        // Also prevents reentry when called from our other thread
+        pthread_mutex_lock(&saver_mutex);
+
+        sprintf(gfx_pid, "%d", graphics_application);
+        getcwd( current_dir, sizeof(current_dir));
+
+        char* argv[4];
+        argv[0] = "gfx_switcher";
+        argv[1] = "-kill_gfx";
+        argv[2] = gfx_pid;
+        argv[3] = 0;
+
+        retval = run_program(
+            current_dir,
+            m_gfx_Switcher_Path,
+            3,
+            argv,
+            0,
+            thePID
+        );
+
+        if (graphics_application) {
+            launchedGfxApp("", 0, -1);
+        }
+        
+        for (i=0; i<200; i++) {
+            boinc_sleep(0.01);      // Wait 2 seconds max
+            // Prevent gfx_switcher from becoming a zombie
+            if (waitpid(thePID, 0, WNOHANG) == thePID) {
+                break;
+            }
+        }
+        pthread_mutex_unlock(&saver_mutex);
     }
     
-    for (i=0; i<200; i++) {
-        boinc_sleep(0.01);      // Wait 2 seconds max
-        // Prevent gfx_switcher from becoming a zombie
-        if (waitpid(thePID, 0, WNOHANG) == thePID) {
-            break;
-        }
-    }
-
-    pthread_mutex_unlock(&saver_mutex);
 #endif
 
 #ifdef _WIN32
@@ -336,15 +419,14 @@ int CScreensaver::terminate_v6_screensaver(GFXAPP_ID& graphics_application) {
 
 
 // Terminate the project (science) graphics application
-// TODO: get rid of 2nd arg
 //
-int CScreensaver::terminate_screensaver(GFXAPP_ID& graphics_application, RESULT *) {
+int CScreensaver::terminate_screensaver(GFXAPP_ID& graphics_application, RESULT* rp) {
     int retval = 0;
 
     if (graphics_application) {
         // V6 Graphics
         if (m_bScience_gfx_running) {
-            terminate_v6_screensaver(graphics_application);
+            terminate_v6_screensaver(graphics_application, rp);
         }
     }
     return retval;
@@ -358,33 +440,58 @@ int CScreensaver::launch_default_screensaver(char *dir_path, GFXAPP_ID& graphics
     int num_args;
     
 #ifdef __APPLE__
-    // For sandbox security, use gfx_switcher to launch default 
-    // gfx app as user boinc_master and group boinc_master.
-    char* argv[6];
-
-    argv[0] = "gfx_switcher";
-    argv[1] = "-default_gfx";
-    argv[2] = THE_DEFAULT_SS_EXECUTABLE;    // Will be changed by gfx_switcher
-    argv[3] = "--fullscreen";
-    argv[4] = 0;
-    argv[5] = 0;
-    if (!m_bConnected) {
-        BOINCTRACE(_T("launch_default_screensaver using --retry_connect argument\n"));
-        argv[4] = "--retry_connect";
-        num_args = 5;
+    if (gIsCatalina) {
+        // As of OS 10.15 (Catalina) screensavers can no longer:
+        //  - launch apps that run setuid or setgid
+        //  - launch apps downloaded from the Internet which have not been 
+        //    specifically approved by the  user via Gatekeeper.
+        // So instead of launching graphics apps via gfx_switcher, we send an 
+        // RPC to the client asking the client to launch them via gfx_switcher.
+        // See comments in gfx_switcher.cpp for a more detailed explanation.
+        // We have tested this on OS 10.13 High Sierra and it works there, too
+        //
+        int thePID = -1;
+        retval = rpc->run_graphics_app("runfullscreen", thePID, gUserName);
+        for (int i=0; i<800; i++) {
+            boinc_sleep(0.01);      // Wait 8 seconds max
+            if (*pid_from_shmem != 0) {
+                graphics_application = *pid_from_shmem;
+                break;
+            }
+        }
+        // fprintf(stderr, "launch_screensaver got pid %d\n", graphics_application);
+        // Inform our helper app what we launched 
+        fprintf(m_gfx_Cleanup_IPC, "%d\n", graphics_application);
+        fflush(m_gfx_Cleanup_IPC);
     } else {
-        num_args = 4;
+        // For sandbox security, use gfx_switcher to launch default 
+        // gfx app as user boinc_master and group boinc_master.
+        char* argv[6];
+
+        argv[0] = "gfx_switcher";
+        argv[1] = "-default_gfx";
+        argv[2] = THE_DEFAULT_SS_EXECUTABLE;    // Will be changed by gfx_switcher
+        argv[3] = "--fullscreen";
+        argv[4] = 0;
+        argv[5] = 0;
+        if (!m_bConnected) {
+            BOINCTRACE(_T("launch_default_screensaver using --retry_connect argument\n"));
+            argv[4] = "--retry_connect";
+            num_args = 5;
+        } else {
+            num_args = 4;
+        }
+
+        retval = run_program(
+            dir_path,
+            m_gfx_Switcher_Path,
+            num_args,
+            argv,
+            0,
+            graphics_application
+        );
     }
-
-    retval = run_program(
-        dir_path,
-        m_gfx_Switcher_Path,
-        num_args,
-        argv,
-        0,
-        graphics_application
-    );
-
+    
     if (graphics_application) {
         launchedGfxApp("boincscr", graphics_application, -1);
     }
@@ -433,7 +540,7 @@ int CScreensaver::terminate_default_screensaver(GFXAPP_ID& graphics_application)
     int retval = 0;
 
     if (! graphics_application) return 0;
-    retval = terminate_v6_screensaver(graphics_application);
+    retval = terminate_v6_screensaver(graphics_application, NULL);
     return retval;
 }
 
@@ -452,7 +559,6 @@ DataMgmtProcType CScreensaver::DataManagementProc() {
     int             retval                      = 0;
     int             suspend_reason              = 0;
     RESULT*         theResult                   = NULL;
-    RESULT*         graphics_app_result_ptr     = NULL;
     RESULT          previous_result;
     // previous_result_ptr = &previous_result when previous_result is valid, else NULL
     RESULT*         previous_result_ptr         = NULL;
@@ -488,10 +594,25 @@ DataMgmtProcType CScreensaver::DataManagementProc() {
     m_bScience_gfx_running = false;
     m_bDefault_gfx_running = false;
     m_bShow_default_ss_first = false;
+    graphics_app_result_ptr = NULL;
 
 #ifdef __APPLE__
     m_vIncompatibleGfxApps.clear();
     default_ss_dir_path = "/Library/Application Support/BOINC Data";
+    if (gIsCatalina) {
+        char shmem_name[MAXPATHLEN];
+        snprintf(shmem_name, sizeof(shmem_name), "/tmp/boinc_ss_%s", gUserName);
+        retval = create_shmem_mmap(shmem_name, sizeof(int), (void**)&pid_from_shmem);
+        // make sure user/group RW permissions are set, but not other.
+        //
+        if (retval == 0) {
+            chmod(shmem_name, 0666);
+            retval = attach_shmem_mmap(shmem_name, (void**)&pid_from_shmem);
+        }
+        if (retval == 0) {
+            *pid_from_shmem = 0;
+        }
+    }
 #else
     default_ss_dir_path = (char*)m_strBOINCInstallDirectory.c_str();
 #endif
@@ -885,6 +1006,19 @@ bool CScreensaver::HasProcessExited(pid_t pid, int &exitCode) {
     int status;
     pid_t p;
     
+    if (gIsCatalina) {
+        // Only the process which launched an app can use waitpid() to test 
+        // whether that app is still running. If we sent an RPC to the client 
+        // asking the client to launch a graphics app via switcher, we must 
+        // send another RPC to the client to call waitpid() for that app.
+        //
+        if (pid_from_shmem) {
+            //fprintf(stderr, "screensaver HasProcessExited got pid_from_shmem = %d\n", *pid_from_shmem);
+            if (*pid_from_shmem != 0) return false;
+        }
+        return true;
+    }
+    
     p = waitpid(pid, &status, WNOHANG);
     exitCode = WEXITSTATUS(status);
     if (p == pid) return true;     // process has exited
